@@ -10,8 +10,8 @@ function flac = fast_ldanaflac(anadir)
 % Original Author: Doug Greve
 % CVS Revision Info:
 %    $Author: greve $
-%    $Date: 2007/12/10 18:53:00 $
-%    $Revision: 1.25.2.1 $
+%    $Date: 2008/08/14 20:16:08 $
+%    $Revision: 1.25.2.4 $
 %
 % Copyright (C) 2002-2007,
 % The General Hospital Corporation (Boston, MA). 
@@ -148,9 +148,9 @@ while(1)
    case '-nextreg',    nextreg     = sscanf(tline,'%*s %d',1);
    case '-rescale',    flac.inorm  = sscanf(tline,'%*s %f',1);
    case '-nskip',      nskip       = sscanf(tline,'%*s %d',1);
-   case '-prestim',    prestim     = sscanf(tline,'%*s %d',1);
-   case '-timewindow', timewindow  = sscanf(tline,'%*s %d',1);
-   case '-ncycles',    ncycles     = sscanf(tline,'%*s %d',1);
+   case '-prestim',    prestim     = sscanf(tline,'%*s %f',1);
+   case '-timewindow', timewindow  = sscanf(tline,'%*s %f',1);
+   case '-ncycles',    ncycles     = sscanf(tline,'%*s %f',1);
    case '-delay',      delay       = sscanf(tline,'%*s %f',1);
    case '-timeoffset', timeoffset  = sscanf(tline,'%*s %f',1);
    case '-fwhm',       sscanf(tline,'%*s %f',1); % dont worry about it
@@ -227,6 +227,7 @@ if(ana.firfit)
 end
 
 flac.ana = ana;
+flac.ana.con = [];
 
 nthev = 1;
 tline = sprintf('EV Baseline baseline nuis');
@@ -330,6 +331,14 @@ if(~isempty(extreg))
   nthev = nthev+1;
 end
 
+
+if(~isempty(flac.tpexcfile))
+  tline = sprintf('EV TExclude texclude nuis %s',flac.tpexcfile);
+  flac.ev(nthev) = flac_ev_parse(tline);
+  nthev = nthev+1;
+end
+
+
 if(strcmp(designtype,'event-related') | strcmp(designtype,'blocked'))
   %-------------- contrasts --------------------------
   tmpstr = sprintf('%s/*.mat',anadir);
